var extLinkPropsPrefUtils = {
	prefNS: "extensions.extlinkprops.",
	prefVer: 1,

	get prefSvc() {
		delete this.prefSvc;
		return this.prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.QueryInterface(Components.interfaces.nsIPrefBranch2);
	},
	pBr: Components.interfaces.nsIPrefBranch,
	ss: Components.interfaces.nsISupportsString,

	init: function() {
		window.addEventListener("unload", this, false);
		const pns = this.prefNS;
		this.prefNSL = pns.length;
		this.prefSvc.addObserver(pns, this, false);
	},
	destroy: function() {
		window.removeEventListener("unload", this, false);
		this.prefSvc.removeObserver(this.prefNS, this);
	},
	handleEvent: function(e) {
		if(e.type == "unload")
			this.destroy();
	},

	// Preferences observer:
	observe: function(subject, topic, pName) {
		if(topic != "nsPref:changed")
			return;
		pName = pName.substr(this.prefNSL);
		["extLinkPropsSvc", "extLinkProps", "extLinkPropsWnd"].forEach(
			function(p) {
				if(!(p in window))
					return;
				var o = window[p];
				if(o.hasOwnProperty("prefsChanged"))
					o.prefsChanged(pName);
			}
		);
	},

	pref: function(pName, pVal) { // Important: unicode strings are not supported!
		pName = this.prefNS + pName;
		if(arguments.length == 1)
			return this.getPref(pName);
		return this.setPref(pName, pVal);
	},
	getPref: function(pName, defaultVal) {
		var pbr = this.pBr;
		switch(this.prefSvc.getPrefType(pName)) {
			case pbr.PREF_STRING: return this.prefSvc.getComplexValue(pName, this.ss).data;
			case pbr.PREF_INT:    return this.prefSvc.getIntPref(pName);
			case pbr.PREF_BOOL:   return this.prefSvc.getBoolPref(pName);
			default:              return defaultVal;
		}
	},
	setPref: function(pName, pVal) {
		var pbr = this.pBr;
		var pType = this.prefSvc.getPrefType(pName);
		var isNew = pType == pbr.PREF_INVALID;
		var vType = typeof pVal;
		if(pType == pbr.PREF_BOOL || (isNew && vType == "boolean"))
			this.prefSvc.setBoolPref(pName, pVal);
		else if(pType == pbr.PREF_INT || (isNew && vType == "number"))
			this.prefSvc.setIntPref(pName, pVal);
		else if(pType == pbr.PREF_STRING || isNew) {
			var ss = this.ss;
			var str = Components.classes["@mozilla.org/supports-string;1"]
				.createInstance(ss);
			str.data = pVal;
			this.prefSvc.setComplexValue(pName, ss, str);
		}
		return pVal;
	},

	prefsMigration: function() {
		const v = this.pref("prefsVersion") || 0;
		if(v >= this.prefVer)
			return false;

		var pSvc = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.QueryInterface(Components.interfaces.nsIPrefBranch2);

		if(v < 1) {
			var pName = "extensions.extendedlink.showhttpheader";
			var pVal = this.getPref(pName, false);
			this.pref("properties.showHttpHeaders", pVal);
			this.pref("download.showHttpHeaders", pVal);
			this.pref("ownWindow.showHttpHeaders", pVal);
			pSvc.deleteBranch(pName);

			pName = "extensions.extendedlink.sizePrecision";
			pVal = this.getPref(pName, 2);
			if(typeof pVal != "number")
				pVal = 2;
			this.pref("sizePrecision", pVal);
			pSvc.deleteBranch(pName);
		}

		this.pref("prefsVersion", this.prefVer);
		pSvc.savePrefFile(null);
		return true;
	}
};

extLinkPropsPrefUtils.init();